/*====================================================================================================*
* 2-Lane Drag Race Timer for 1/10 Scale RC cars                            Version 5.3 - 20 Aug. 2022 *
* Created by: Bill van Leeuwen (email: Billv923@outlook.com) for the Arduino MEGA microcontroller     *
* Version 5 provides the following features:                                                          *
*   1. Bracket racing capabilty                                                                       *
*   2. Christmas Tree Control including:                                                              *
*      a. Selectable Standard, Pro or Street Outlaw light sequencing                                  *
*      b. Staging & Disqualification (DQ) (red-light) indicators                                      *
*   3. Lane Sensors at the following positions:                                                       *
*      a. Pre-Stage, Stage & Guard sensors                                                            *
*      b. 6 Ft down the track sensors                                                                 *
*      c. 66 Ft down the track sensors (half way)                                                     *
*      d. Trap sensors 5.28 Ft in front of Finish Line sensors                                        *
*      e. Finish Line sensors (132 ft)                                                                *
*   4. Time accuracy to 3 decimal places (0.001 seconds)                                              *
*   5. Manual and Auto race start capability                                                          *
*   6. Hand-held pushbutton timer control capability                                                  *
*   7. Winner indication on corresponding Xmas Tree stage lamp indicators                             *
*   8. Sensor fault (not ready) indication & reporting                                                *
*   9. User selectable 1 or 2 racer operation                                                         *
*                                                                                                     *
* This version is designed to work in conjunction with the author's race management software          *
* RC_DRT_RaceManager_Ver5b.pde which runs on a PC attached to the Arduino MEGA via the USB port. This *
* version has been designed to work with the Autonics(tm) BJ15M-TDT through beam infrared sensors but *
* will work with other sensors having similar functionality, performance and output.                  *
*                                                                                                     *
* This program measures the start reaction time, 6 Ft & 66 Ft marker time, trap sensor time and finish*
* time for 2 lanes. The trap time is used to calculate the vehicle speed during the last 5.28 feet of *
* the track. Race status and times are sent to the serial port to interface with the race management  * 
* software running on the PC. Times are measured from when the racer leaves the start line to         *
* activation of the various sensors of each lane. Each racer has 10 seconds to finish the race. If a  *
* racer doesn't cross the finish line within that time, it is assumed that the racer had a problem or *
* the lane was not used. In either case, the value 9999.999 is assigned to the lane time as an error  *
* indication for use by the race management software. A hand-held pushbutton switch is provided to    *
* remotely start the Xmas Tree countdown sequence or to reset the timer for the next race.            *
*                                                                                                     *
* Refer to the Schematic Diagram, parts list and design notes that accompany this software for more   *
* information. Feel free to contact the author at Billv923@outlook.com if any questions               *
*====================================================================================================*/

//Initialization
//Race State Definitions
#define NotReady       0              //Race State = Not Ready
#define Ready          1              //Race State = Ready
#define Staged         2              //Race State = Staged
#define Countdown      3              //Race State = Count Down Sequence in progress
#define Racing         4              //Race State = Racing
#define Finished       5              //Race State = Finished

//Pin Assignments - Misc
#define hand_sw       10              //Manual hand-held switch - Digital Input Pin D10
//Pin Assignments - Xmas Tree Indicators
#define XtreePstg1    22              //Xmas Tree Lane1 Pre-stage Indicator - Digital Output Pin D22
#define XtreeStg1     24              //Xmas Tree Lane1 Stage Indicator - Digital Output Pin D24
#define XtreeSeq1a    26              //Xmas Tree Lane1 Amber Sequence Indicator #1 - Digital Output Pin D26
#define XtreeSeq1b    28              //Xmas Tree Lane1 Amber Sequence Indicator #2 - Digital Output Pin D28
#define XtreeSeq1c    30              //Xmas Tree Lane1 Amber Sequence Indicator #3 - Digital Output Pin D30
#define XtreeGrn1     32              //Xmas Tree Lane1 Green Go Indicator - Digital Output Pin D32
#define XtreeDsq1     34              //Xmas Tree Lane1 Disqualification Indicator - Digital Output Pin D34
#define XtreeSpr1     36              //Xmas Tree Lane1 Spare relay RM1K8 - Digital Output Pin D36
#define XtreePstg2    23              //Xmas Tree Lane2 Pre-stage Indicator - Digital Output Pin D23
#define XtreeStg2     25              //Xmas Tree Lane2 stage Indicator - Digital Output Pin D25
#define XtreeSeq2a    27              //Xmas Tree Lane2 Amber Sequence Indicator #1 - Digital Output Pin D27
#define XtreeSeq2b    29              //Xmas Tree Lane2 Amber Sequence Indicator #2 - Digital Output Pin D29
#define XtreeSeq2c    31              //Xmas Tree Lane2 Amber Sequence Indicator #3 - Digital Output Pin D31
#define XtreeGrn2     33              //Xmas Tree Lane2 Green Go Indicator - Digital Output Pin D33
#define XtreeDsq2     35              //Xmas Tree Lane2 Disqualification Indicator - Digital Output Pin D35
#define XtreeSpr2     37              //Xmas Tree Lane2 Spare relay RM2K8 - Digital Output Pin D37
//Pin Assignments - Lane Sensor Inputs
#define PStg_Sense1   40              //Lane1 Pre-Stage Sensor - Digital Input Pin D40
#define PStg_Sense2   41              //Lane2 Pre-Stage Sensor - Digital Input Pin D41
#define Stg_Sense1    42              //Lane1 Stage Sensor - Digital Input Pin D42
#define Stg_Sense2    43              //Lane2 Stage Sensor - Digital Input Pin D43
#define Grd_Sense1    44              //Lane1 Guard Sensor - Digital Input Pin D44
#define Grd_Sense2    45              //Lane2 Guard Sensor - Digital Input Pin D45
#define T06_Sense1    46              //Lane1 6 Foot Marker Sensor - Digital Input Pin D46
#define T06_Sense2    47              //Lane2 6 Foot Marker Sensor - Digital Input Pin D47
#define T66_Sense1    48              //Lane1 66 Foot Marker Sensor - Digital Input Pin D48
#define T66_Sense2    49              //Lane2 66 Foot Marker Sensor - Digital Input Pin D49
#define Trap_Sense1   50              //Lane1 Trap Sensor - Digital Input Pin D50
#define Trap_Sense2   51              //Lane2 Trap Sensor - Digital Input Pin D51
#define Fin_Sense1    52              //Lane1 Finish Sensor - Digital Input Pin D52
#define Fin_Sense2    53              //Lane2 Finish Sensor - Digital Input Pin D53

//Variable Initialization
boolean Hdcap_Flag;                   //True when handicap (bracket) racing mode in effect
boolean Lane1_ready;                  //False if any Lane1 sensor is obstructed
boolean Lane2_ready;                  //False if any Lane2 sensor is obstructed
boolean Track_ready;                  //False if track is not ready
boolean Breakout_Flag[2];             //True if Lane breakout (E.T. < Dial-in)
boolean Stg1_Flag;                    //True if Lane1 stage sensor was tripped
boolean Stg2_Flag;                    //True if Lane2 stage sensor was tripped
boolean Grd1_Flag;                    //True if Lane1 guard sensor was tripped
boolean Grd2_Flag;                    //True if Lane2 guard sensor was tripped
boolean T06L1_Flag;                   //True if Lane1 6 Ft track sensor was tripped
boolean T06L2_Flag;                   //True if Lane2 6 Ft track sensor was tripped
boolean T66L1_Flag;                   //True if Lane1 66 Ft track sensor was tripped
boolean T66L2_Flag;                   //True if Lane2 66 Ft track sensor was tripped
boolean Trap1_Flag;                   //True if Lane1 Trap sensor was tripped
boolean Trap2_Flag;                   //True if Lane2 Trap sensor was tripped
boolean Fin1_Flag;                    //True if Lane1 Finish sensor was tripped
boolean Fin2_Flag;                    //True if Lane2 Finish sensor was tripped
boolean L1blink_Flag;                 //True if Lane1 Stage indicator is commanded to blink
boolean L2blink_Flag;                 //True if Lane2 Stage indicator is commanded to blink
boolean L1stage_Flag;                 //True if Lane1 vehicle is staged
boolean L2stage_Flag;                 //True if Lane2 vehicle is staged
boolean Hold_Flag;                    //True if 'Hold' command rec'd from PC to disable start switch
boolean Autostart_Flag;               //True if 'Auto-start' command rec'd from PC to auto start timer when staged
boolean OneRacer_Flag;                //True if timer set to handle a single racer on Lane1
boolean Win_Flag[2];                  //Where true=winner
boolean Grd_Enable;                   //Set to true when using guard beams
int i,n;                              //Working variables
int Foul_Flag[2];                     //Equals 1 if 1'st to foul, 2 if 2'nd to foul
int XTmode;                           //Xmas tree sequence mode (0=Std mode, 1=Pro mode, 2=Outlaw mode)
int lane_place[2];                    //Lane finish place array
int Ln1_Seq_State;                    //Progression state of Lane1 Xmas Tree sequencing lights (0,1,2,3,4,5)
int Ln2_Seq_State;                    //Progression state of Lane2 Xmas Tree sequencing lights (0,1,2,3,4,5)
                                      //Where: 0=start, 1=Amber1, 2=Amber2, 3=Amber3, 4=Green, 5=Off
int trackstate;                       //0=NotReady, 1=Ready, 2=Countdown, 3=Racing, 4=Finished
int Ln1_State;                        //Lane1: 0=NotReady, 1=Ready, 2=Countdown, 3=Racing, 4=Finished
int Ln2_State;                        //Lane2: 0=NotReady, 1=Ready, 2=Countdown, 3=Racing, 4=Finished
int Num_Vehicles;                     //Number of vehicles racing (active lanes): 1 or 2
int Dialin1;                          //Lane1 dial in time (milliseconds)
int Dialin2;                          //Lane2 dial in time (milliseconds)
int Dialin_Dly1;                      //Lane1 timing start delay (milliseconds)
int Dialin_Dly2;                      //Lane2 timing start delay (milliseconds)
unsigned long max_duration = 10000;   //Default race duration in milliseconds
unsigned long Start_time[2];          //Time when Lane1 & Lane2 green light illuminates (milliseconds)
unsigned long Exp_Time[2];            //Lane1 & Lane2 expiration Time (milliseconds)
unsigned long Clk_Time;               //CPU Clock Time (milliseconds)
unsigned long Xtime;                  //Multi-use time variable (milliseconds)
unsigned long Seq_Time1;              //Time variable used in Lane1 Xtree Sequence routine
unsigned long Seq_Time2;              //Time variable used in Lane1 Xtree Sequence routine
unsigned long React_time[2];          //Reaction time data array (milliseconds)
unsigned long T06_time[2];            //6 Ft Marker time data array (milliseconds)
unsigned long T66_time[2];            //66 Ft Marker time data array (milliseconds)
unsigned long Trap_time[2];           //Trap time data array (milliseconds)
unsigned long Fin_time[2];            //Elapsed finish time data array (milliseconds)
unsigned long blinkTime;              //Time variable used in winning lane blink routine
float Time;                           //Working variable - Calculated Lane Time (seconds)
float Temp;                           //Working temporary floating point variable
float Rtime[2];                       //Reaction time data array (seconds)
float Etime[2];                       //Elapsed time data array (seconds)
float Otime[2];                       //Overall time data array (seconds)
float Speed[2];                       //Speed data array (mph)
float Margin[2];                      //Used in handicap win determination (Elapsed time - Dial-in0
char SMsg0[4] = "NRD";                //Serial message - Not Ready
char SMsg1[4] = "RDY";                //Serial message - Ready
char SMsg2[4] = "STG";                //Serial message - Staged
char SMsg3[4] = "CNT";                //Serial message - Countdown Sequence underway
char SMsg4[4] = "RAC";                //Serial message - Racing
char SMsg5[4] = "FIN";                //Serial message - Finished
char SMsg7[4] = "TRK";                //Serial message - Track Sensor(s) blocked
char SMsg8[4] = "RST";                //Serial message - Manual RESET executed
String serial_data = "";              //Serial data input string
String error_codes = "";              //Message string populated with error codes
String Str1;                          //Working string variable

//======================================= SETUP ROUTINE =======================================
void setup() {  
//Hardware Setup
  //Initialize Xmas Tree indicator pins as outputs; track sensors and Reset switch pins as inputs
  pinMode(XtreePstg1, OUTPUT);        //Xmas Tree Lane1 white Pre-stage Indicator
  pinMode(XtreeStg1, OUTPUT);         //Xmas Tree Lane1 white Stage Indicator
  pinMode(XtreeSeq1a, OUTPUT);        //Xmas Tree Lane1 Amber Sequence Indicator #1
  pinMode(XtreeSeq1b, OUTPUT);        //Xmas Tree Lane1 Amber Sequence Indicator #2
  pinMode(XtreeSeq1c, OUTPUT);        //Xmas Tree Lane1 Amber Sequence Indicator #3
  pinMode(XtreeGrn1, OUTPUT);         //Xmas Tree Lane1 Green Go Indicator
  pinMode(XtreeDsq1, OUTPUT);         //Xmas Tree Lane1 Disqualification Indicator
  pinMode(XtreeSpr1, OUTPUT);         //Xmas Tree Lane1 Spare relay RM1-K8
  pinMode(XtreePstg2, OUTPUT);        //Xmas Tree Lane2 white Pre-stage Indicator
  pinMode(XtreeStg2, OUTPUT);         //Xmas Tree Lane2 white Stage Indicator
  pinMode(XtreeSeq2a, OUTPUT);        //Xmas Tree Lane2 Amber Sequence Indicator #1
  pinMode(XtreeSeq2b, OUTPUT);        //Xmas Tree Lane2 Amber Sequence Indicator #2
  pinMode(XtreeSeq2c, OUTPUT);        //Xmas Tree Lane2 Amber Sequence Indicator #3
  pinMode(XtreeGrn2, OUTPUT);         //Xmas Tree Lane2 Green Go Indicator
  pinMode(XtreeDsq2, OUTPUT);         //Xmas Tree Lane2 Disqualification Indicator
  pinMode(XtreeSpr2, OUTPUT);         //Xmas Tree Lane2 Spare relay RM2-K8
  
  //Note: Sensor inputs transition from LOW to HIGH when light beam is interrupted
  pinMode(PStg_Sense1,INPUT_PULLUP);  //Lane1 Pre-tage sensor
  pinMode(PStg_Sense2,INPUT_PULLUP);  //Lane2 Pre-tage sensor
  pinMode(Stg_Sense1,INPUT_PULLUP);   //Lane1 Stage sensor
  pinMode(Stg_Sense2,INPUT_PULLUP);   //Lane2 Stage sensor
  pinMode(Grd_Sense1,INPUT_PULLUP);   //Lane1 Guard sensor
  pinMode(Grd_Sense2,INPUT_PULLUP);   //Lane2 Guard sensor
  pinMode(T06_Sense1,INPUT_PULLUP);   //Lane1 6 ft track sensor
  pinMode(T06_Sense2,INPUT_PULLUP);   //Lane2 6 ft track sensor
  pinMode(T66_Sense1,INPUT_PULLUP);   //Lane1 66 ft track sensor
  pinMode(T66_Sense2,INPUT_PULLUP);   //Lane2 66 ft track sensor
  pinMode(Trap_Sense1,INPUT_PULLUP);  //Lane1 Trap sensor
  pinMode(Trap_Sense2,INPUT_PULLUP);  //Lane2 Trap sensor
  pinMode(Fin_Sense1,INPUT_PULLUP);   //Lane1 Finish line sensor
  pinMode(Fin_Sense2,INPUT_PULLUP);   //Lane2 Finish line sensor
  pinMode(hand_sw,INPUT_PULLUP);      //Switch is normally HIGH & transitions to LOW when pressed

  digitalWrite(XtreeSpr1, HIGH);      //Turn spare relay RM1-K8 off
  digitalWrite(XtreeSpr2, HIGH);      //Turn spare relay RM2-K8 off
  
//Software Setup
  Turnoff_All_Lights();               //Call subroutine to turn off Xmas Tree lights
  Serial.begin(9600);                 //Enable serial comms to PC at 9600 baud
  trackstate = NotReady;              //Set initial track state to not ready
  Ln1_State = NotReady;               //Set Lane1 initial race state to not ready
  Ln2_State = NotReady;               //Set Lane1 initial race state to not ready
  Foul_Flag[0] = 0;                   //Preset Lane1 foul flag to 0
  Foul_Flag[1] = 0;                   //Preset Lane2 foul flag to 0
  XTmode = 0;                         //Set Xmas tree sequence to Std mode
  Autostart_Flag=false;               //Preset Autostart flag to "manual" start
  OneRacer_Flag=false;                //Preset one racer flag to false
  Num_Vehicles = 2;                   //Preset number of vehicles racing to 2
  Grd_Enable = true;                  //Enable Guard Beam capability if they're used
}

//======================================== MAIN LOOP ============================================
void loop() {
  //CHECK & PROCESS INCOMING SERIAL DATA
  if (Serial.available() > 0) {
    serial_data = Serial.readStringUntil('\n');
    Serial.flush();
    if (serial_data.indexOf('R') >= 0) {            //Check for Reset request from PC
      timer_reset();                                //Call subroutine to reset the timer
    }
    if (serial_data.indexOf('S') >= 0) {            //Check for race start request from PC
      if (trackstate == Staged) {                   //Check if vehicles are staged
        Set_Dialin_Delays();                        //Call subroutine to set dial-in delays
        trackstate = Countdown;                     //Set race state to Countdown mode
        Ln1_State = Countdown;                      //Set Lane1 state to Countdown state
        if (!OneRacer_Flag) Ln2_State = Countdown;  //Set Lane2 state to Countdown state
        Serial.println(SMsg3);                      //Send Countdown state message to PC
      }
    }
    if (serial_data.indexOf('H') >= 0) {            //Check for hold request from PC
      Hold_Flag = true;                             //Set Hold flag
      Serial.println("@");                          //Send acknowledge message to PC
    }
    if (serial_data.indexOf('C') >= 0) {            //Check for continue request from PC
      Hold_Flag = false;                            //Clear Hold flag
      Serial.println("@");                          //Send acknowledge message to PC
    }
    if (serial_data.indexOf("LT") >= 0) {           //Check for Xmas tree lamp test request from PC
      Lamptest();                                   //Call Xmas Tree Lamp Test subroutine
      Serial.println("@");                          //Send Acknowledge message to PC
    }
    if (serial_data.indexOf("M0") >= 0) {           //Check for Xmas Tree "Std" mode request from PC
      XTmode = 0;                                   //Set Xmas Tree to Std Mode if true
      Serial.println("@");                          //Send Acknowledge message to PC
    }
    if (serial_data.indexOf("M1") >= 0) {           //Check for Xmas Tree "Pro" mode request from PC
      XTmode = 1;                                   //Set Xmas Tree to Std Mode if true
      Serial.println("@");                          //Send Acknowledge message to PC
    }
    if (serial_data.indexOf("M2") >= 0) {           //Check for Xmas Tree "Outlaw" mode request from PC
      XTmode = 2;                                   //Set Xmas Tree to Std Mode if true
      Serial.println("@");                          //Send Acknowledge message to PC
    }
    if (serial_data.indexOf("A1") >= 0) {           //Check for timer auto-start request from PC
      Autostart_Flag = true;                        //Set timer to Auto-start mode
      Serial.println("@");                          //Send acknowledge message to PC
    }
    if (serial_data.indexOf("A0") >= 0) {           //Check for timer manual-start request from PC
      Autostart_Flag = false;                       //Set timer to Manual-start mode
      Serial.println("@");                          //Send acknowledge message to PC
    }
    if (serial_data.indexOf("P1") >= 0) {           //Check for single racer request from PC
      OneRacer_Flag = true;                         //Set timer to single racer mode
      Num_Vehicles = 1;                             //Set number of vehicles racing to 1
      Serial.println("@");                          //Send acknowledge message to PC
    }
    if (serial_data.indexOf("P2") >= 0) {           //Check for dual racer request from PC
      OneRacer_Flag = false;                        //Set timer to dual racer mode
      Num_Vehicles = 2;                             //Set number of vehicles racing to 2
      Serial.println("@");                          //Send acknowledge message to PC
    }
    if (serial_data.indexOf("D1:") >= 0) {          //Check for receipt of Lane1 dial-in time from PC
      //Input string format: D1:xxxxx where xxxxx is Lane1 dial-in time in milliseconds
      //Valid range: 00000 - 09999 (0.0 to 9.999 seconds)
      if (serial_data.length() == 8) {              //Check integrity of input string
        Str1 = serial_data.substring(3, 8);         //Extract Lane1 dial-in time
        Dialin1 = Str1.toInt();                     //And convert to integer
        Serial.println("@");                        //Send acknowledge message to PC
      }
    }
    if (serial_data.indexOf("D2:") >= 0) {          //Check for receipt of Lane2 dial-in time from PC
      //Input string format: D2:xxxxx where xxxxx is Lane2 dial-in time in milliseconds
      //Valid range: 00000 - 09999 (0.0 to 9.999 seconds)
      if (serial_data.length() == 8) {              //Check integrity of input string
        Str1 = serial_data.substring(3, 8);         //Extract Lane2 dial-in time
        Dialin2 = Str1.toInt();                     //And convert to integer
        Serial.println("@");                        //Send acknowledge message to PC
      }
    }
    //max_duration
    if (serial_data.indexOf("TO:") >= 0) {          //Check for receipt of timer timeout duration from PC
      //Input string format: TO:xx where xx is the timeout duration in seconds
      //Valid range: 05 - 30
      if (serial_data.length() == 5) {              //Check integrity of input string
        Str1 = serial_data.substring(3, 5);         //Extract timeout duration in seconds
        max_duration = Str1.toInt() * 1000;         //And convert to milliseconds
        Serial.println("@");                        //Send acknowledge message to PC
      }
    }
  }

  //CHECK & PROCESS LANE 1 STAGING SENSORS
  if (digitalRead(PStg_Sense1)==1) {                //Is Lane1 Pre-Stage sensor tripped?
    digitalWrite(XtreePstg1,LOW);                   //Turn on Lane1 Pre-Stage Indicator
  }
  else digitalWrite(XtreePstg1,HIGH);               //Turn off Lane1 Pre-Stage Indicator
  if (digitalRead(Stg_Sense1)==1) {                 //Is Lane1 Stage sensor tripped?
    digitalWrite(XtreeStg1,LOW);                    //Turn on Lane1 Stage Indicator
    Stg1_Flag = true;                               //Set Lane1 as staged
  }
  else {                                            //Lane1 Stage sensor not tripped
    digitalWrite(XtreeStg1,HIGH);                   //Turn off Lane1 Stage Indicator
    Stg1_Flag = false;                              //Set Lane1 as not staged
  }
  //CHECK & PROCESS LANE 2 STAGING SENSORS
  if (digitalRead(PStg_Sense2)==1) {                //Is Lane2 Pre-Stage sensor tripped?
    digitalWrite(XtreePstg2,LOW);                   //Turn on Lane2 Pre-Stage Indicator
  }
  else digitalWrite(XtreePstg2,HIGH);               //Turn off Lane2 Pre-Stage Indicator
  if (digitalRead(Stg_Sense2)==1) {                 //Is Lane2 Stage sensor tripped?
    digitalWrite(XtreeStg2,LOW);                    //Turn on Lane2 Stage Indicator
    Stg2_Flag = true;                               //Set Lane2 as staged
  }
  else {                                            //Lane2 Stage sensor not tripped
    digitalWrite(XtreeStg2,HIGH);                   //Turn off Lane2 Stage Indicator
    Stg2_Flag = false;                              //Set Lane2 as not staged
  }

  //CHECK & PROCESS LANE 1 & 2 GAURD SENSORS IF THEY'RE USED
  if (Grd_Enable) {                                 //Do if Guard Sensor feature enabled?
    if (digitalRead(Grd_Sense1)==1) {               //Is Lane1 Guard sensor tripped?
      Grd1_Flag = true;                             //Set Lane1 Guard as tripped
    }
    else Grd1_Flag = false;                         //Set Lane1 Guard as not tripped
    if (digitalRead(Grd_Sense2)==1) {               //Is Lane2 Guard sensor tripped?
      Grd2_Flag = true;                             //Set Lane2 Guard as tripped
    }
    else Grd2_Flag = false;                         //Set Lane2 Guard as not tripped
    if (trackstate==Ready || trackstate == Staged) {
      if (Grd1_Flag) digitalWrite(XtreeDsq1,LOW);   //Turn on Lane1 red DQ light
        else digitalWrite(XtreeDsq1,HIGH);          //Turn off Lane1 red DQ light
      if (Grd2_Flag) digitalWrite(XtreeDsq2,LOW);   //Turn on Lane2 red DQ light
        else digitalWrite(XtreeDsq2,HIGH);          //Turn off Lane2 red DQ light
    }
  }
 
  //READY STATE
  if (trackstate == Ready) {                        //Do if track in Ready race state
    if (OneRacer_Flag) {                            //Do if only 1 car racing on Lane1
      if (Stg1_Flag && !Grd1_Flag) {                //Is Lane1 vehicle staged?
        delay(250);                                 //Wait 1/4 second
        if (digitalRead(Stg_Sense1)==1) {           //Is Lane1 vehicle still staged
          Ln1_State = Staged;                       //Set Lane1 race state to Staged if so
          trackstate = Staged;                      //Set track race state to Staged if so
          L1stage_Flag = true;                      //Set the Lane1 staged flag
          Serial.println(SMsg2);                    //Send Staged state message to PC
        }
      }
    }
    else {                                          //Do if 2 cars are racing
      if (Stg1_Flag && !Grd1_Flag && Stg2_Flag && !Grd2_Flag) { //Are both vehicles staged?
        delay(250);                                 //Wait 1/4 second
        if (digitalRead(Stg_Sense1)==1 && digitalRead(Stg_Sense2)==1) { //Are both vehicles still staged?
          Ln1_State = Staged;                       //Set Lane1 race state to Staged if so
          Ln2_State = Staged;                       //Set Lane2 race state to Staged if so
          trackstate = Staged;                      //Set track race state to Staged if so
          L1stage_Flag = true;                      //Set the Lane1 staged flag
          L2stage_Flag = true;                      //Set the Lane2 staged flag
          Serial.println(SMsg2);                    //Send Staged state message to PC
        }
      }
    }
  }

  //STAGED STATE
  if (trackstate == Staged) {                       //Do if track is in the Staged state
    if (!Hold_Flag) {                               //Disable race start if Hold flag is set
      if (Autostart_Flag) {                         //Do auto start if flag is set
        delay(random(500, 1500));                   //Random wait between 0.5 and 1.5 sec before we start countdown
        Set_Dialin_Delays();                        //Call subroutine to set dial-in delays
        trackstate = Countdown;                     //Set track state to Countdown state
        Ln1_State = Countdown;                      //Set Lane1 state to Countdown state
        if (!OneRacer_Flag) Ln2_State = Countdown;  //Set Lane2 state to Countdown state
        Serial.println(SMsg3);                      //Send Countdown state message to PC
      }
      else {                                        //Do manual start if Autostart_Flag is not set
        while(digitalRead(hand_sw)==0) {            //Do while hand switch is pressed
          if (millis() > (Clk_Time + 500)) {        //Has switch been held for 1/2 second?
            Set_Dialin_Delays();                    //Call subroutine to set dial-in delays
            trackstate = Countdown;                 //Set track state to Countdown
            Ln1_State = Countdown;                  //Set Lane1 state to Countdown state
            if (!OneRacer_Flag) Ln2_State = Countdown; //Set Lane2 state to Countdown state
            Serial.println(SMsg3);                  //Send Countdown state message to PC
            break;                                  //exit while loop
          }
          delay(50);                                //Wait awhile before we loop again
        }
      }
    }
    //The following resets timer to ready state if a car un-stages before the countdown begins
    if (OneRacer_Flag) {                            //If 1 racer
      if (!Stg1_Flag || Grd1_Flag) {                //Is Lane1 vehicle no longer staged?
        trackstate = Ready;                         //Revert back to Ready state
        Serial.println(SMsg1);                      //Send Ready state message to PC
      }
    }
    else {                                          //If 2 racers
      if (!Stg1_Flag || Grd1_Flag || !Stg2_Flag || Grd2_Flag) { //Is either Lane1 or Lane2 no longer staged?
        trackstate = Ready;                         //Revert back to Ready state
        Serial.println(SMsg1);                      //Send Ready state message to PC
      }
    }
  }
  
  //NOTE: Clock time is used extensively when the timer is in the countdown & racing states.
  //      So for speed & efficiency we get the current time just once each pass through the loop
  Clk_Time = millis();                              //Get CPU clock time
  
  //LANE 1 COUNTDOWN SEQUENCE
  if (Ln1_State == Countdown) {                     //Do if Lane1 in Countdown state
    if (XTmode == 0) {                              //Do standard Xmas Tree lamp sequence
      switch (Ln1_Seq_State) {                      //Get Lane1 sequence state
        case 0:
          Seq_Time1 = Clk_Time + Dialin_Dly1;       //Set dial-in delay for next sequence
          Ln1_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:                                     //Turn on first amber light
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Amber1 light
            if (Foul_Flag[0]!=1) digitalWrite(XtreeSeq1a,LOW); //Turn on Lane1 Amber Indicator #1 if not DQ'd
            Seq_Time1 = Seq_Time1 + 400;            //Add another 0.4 sec delay for next sequence
            Ln1_Seq_State = 2;                      //Set Sequence State count to 2 (Amber2 on)
          }
          break;
        case 2:                                     //Turn on second amber light
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Amber2 light
            digitalWrite(XtreeSeq1a,HIGH);          //Turn off Lane1 Amber Sequence Indicator #1
            if (Foul_Flag[0]!=1) digitalWrite(XtreeSeq1b,LOW); //Turn on Lane1 Amber Indicator #2 if not DQ'd
            Seq_Time1 = Seq_Time1 + 400;            //Add another 0.4 sec delay for next sequence
            Ln1_Seq_State = 3;                      //Set Sequence State count to 3 (Amber3 on)
          }
          break;
        case 3:                                     //Turn on third amber light
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Amber3 light
            digitalWrite(XtreeSeq1b,HIGH);          //Turn off Lane1 Amber Sequence Indicator #2
            if (Foul_Flag[0]!=1) digitalWrite(XtreeSeq1c,LOW); //Turn on Lane1 Amber Indicator #3 if not DQ'd
            Seq_Time1 = Seq_Time1 + 400;            //Add another 0.4 sec delay for next sequence
            Ln1_Seq_State = 4;                      //Set Sequence State count to 4 (Green on)
          }
          break;
        case 4:                                     //Turn on green light
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Green light
            digitalWrite(XtreeSeq1c,HIGH);          //Turn off Lane1 Amber Sequence Indicator #3
            if (Foul_Flag[0]!=1) digitalWrite(XtreeGrn1,LOW); //Turn on Lane1 Green Go Indicator if not DQ'd
            Ln1_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln1_State = Racing;                     //Set Lane1 race state to Racing
            Start_time[0] = Clk_Time;               //Set Lane1 race start time
            Exp_Time[0] = Clk_Time + max_duration;  //Set Lane1 expiration time till timeout
          }
          break;
        default:
          //Do nothing
          break;
      }
    }
    if (XTmode == 1) {                              //Do Pro Xmas Tree lamp sequence
      switch (Ln1_Seq_State) {                      //Get Lane1 sequence state
        case 0:
          Seq_Time1 = Clk_Time + Dialin_Dly1;       //Set dial in delay for next sequence
          Ln1_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:                                     //Turn on all amber lights
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Amber lights
            if (Foul_Flag[0]!=1) {                  //Do if Lane1 hasn't DQ'd
              digitalWrite(XtreeSeq1a,LOW);         //Turn on Lane1 Amber Sequence Indicator #1
              digitalWrite(XtreeSeq1b,LOW);         //Turn on Lane1 Amber Sequence Indicator #2
              digitalWrite(XtreeSeq1c,LOW);         //Turn on Lane1 Amber Sequence Indicator #3
            }
            Seq_Time1 = Seq_Time1 + 400;            //Add another 400 ms delay for next sequence
            Ln1_Seq_State = 2;                      //Set Sequence State count to 2
          }
          break;
        case 2:                                     //Turn off amber lights & turn on green light
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Green light
            digitalWrite(XtreeSeq1a,HIGH);          //Turn off Lane1 Amber Sequence Indicator #1
            digitalWrite(XtreeSeq1b,HIGH);          //Turn off Lane1 Amber Sequence Indicator #2
            digitalWrite(XtreeSeq1c,HIGH);          //Turn off Lane1 Amber Sequence Indicator #3
            if (Foul_Flag[0]!=1) digitalWrite(XtreeGrn1,LOW); //Turn on Lane1 Green Go Indicator if not DQ'd
            Ln1_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln1_State = Racing;                     //Set Lane1 race state to Racing
            Start_time[0] = Clk_Time;               //Set Lane1 race start time
            Exp_Time[0] = Clk_Time + max_duration;  //Set Lane1 expiration time till timeout
          }
          break;
        default:
          //Do nothing
          break;
      }
    }
    if (XTmode == 2) {                              //Do Outlaw Xmas Tree lamp sequence
      switch (Ln1_Seq_State) {                      //Get Lane1 sequence state
        case 0:
          Seq_Time1 = Clk_Time + Dialin_Dly1;       //Set dial in delay for next sequence
          Ln1_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:
          if (Clk_Time > Seq_Time1) {               //Check if time to turn on Green light
            if (Foul_Flag[0]!=1) digitalWrite(XtreeGrn1,LOW); //Turn on Lane1 Green Go Indicator if not DQ'd
            Ln1_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln1_State = Racing;                     //Set Lane1 race state to Racing
            Start_time[0] = Clk_Time;               //Set Lane1 race start time
            Exp_Time[0] = Clk_Time + max_duration;  //Set Lane1 expiration time till timeout
          }
          break;
        default:
          //Do nothing
          break;
      }
    }
    if (Foul_Flag[0]==0 && (!Stg1_Flag || Grd1_Flag)) { //Did Lane1 vehicle leave early during countdown?
      React_time[0] = Clk_Time;                     //Get Lane1 reaction time
      Foul_Flag[0] = Foul_Flag[1] + 1;              //Set Lane1 foul flag as 1'st or 2'nd to foul
      if (Foul_Flag[0]==1) digitalWrite(XtreeDsq1,LOW); //Only red-light Lane1 if fouled first (DQ'd)
    }
  }
  //LANE 2 COUNTDOWN SEQUENCE
  if (!OneRacer_Flag && Ln2_State==Countdown) {     //Do if Lane2 in Countdown state
    if (XTmode == 0) {                              //Do standard Xmas Tree lamp sequence
      switch (Ln2_Seq_State) {                      //Get Lane2 sequence state
        case 0:
          Seq_Time2 = Clk_Time + Dialin_Dly2;       //Set dial-in delay for next sequence
          Ln2_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:                                     //Turn on first amber light
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Amber1 light
            if (Foul_Flag[1]!=1) digitalWrite(XtreeSeq2a,LOW); //Turn on Lane2 Amber Indicator #1 if not DQ'd
            Seq_Time2 = Seq_Time2 + 400;            //Add another 0.4 sec delay for next sequence
            Ln2_Seq_State = 2;                      //Set Sequence State count to 2 (Amber2 on)
          }
          break;
        case 2:                                     //Turn on second amber light
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Amber2 light
            digitalWrite(XtreeSeq2a,HIGH);          //Turn off Lane2 Amber Sequence Indicator #1
            if (Foul_Flag[1]!=1) digitalWrite(XtreeSeq2b,LOW); //Turn on Lane2 Amber Indicator #2 if not DQ'd
            Seq_Time2 = Seq_Time2 + 400;            //Add another 0.4 sec delay for next sequence
            Ln2_Seq_State = 3;                      //Set Sequence State count to 3 (Amber3 on)
          }
          break;
        case 3:                                     //Turn on third amber light
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Amber3 light
            digitalWrite(XtreeSeq2b,HIGH);          //Turn off Lane2 Amber Sequence Indicator #2
            if (Foul_Flag[1]!=1) digitalWrite(XtreeSeq2c,LOW); //Turn on Lane2 Amber Indicator #3 if not DQ'd
            Seq_Time2 = Seq_Time2 + 400;            //Add another 0.4 sec delay for next sequence
            Ln2_Seq_State = 4;                      //Set Sequence State count to 4 (Green on)
          }
          break;
        case 4:                                     //Turn on green light
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Green light
            digitalWrite(XtreeSeq2c,HIGH);          //Turn off Lane2 Amber Sequence Indicator #3
            if (Foul_Flag[1]!=1) digitalWrite(XtreeGrn2,LOW); //Turn on Lane2 Green Go Indicator if not DQ'd
            Ln2_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln2_State = Racing;                     //Set Lane2 race state to Racing
            Start_time[1] = Clk_Time;               //Set Lane2 race start time
            Exp_Time[1] = Clk_Time + max_duration;  //Set Lane2 expiration time till timeout
          }
          break;
        default:
          //Do nothing
          break;
      }
    }
    if (XTmode == 1) {                              //Do Pro Xmas Tree lamp sequence
      switch (Ln2_Seq_State) {
        case 0:
          Seq_Time2 = Clk_Time + Dialin_Dly2;       //Set dial in delay for next sequence
          Ln2_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:                                     //Turn on all amber lights
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Amber lights
            if (Foul_Flag[1]!=1) {                  //Do if Lane2 hasn't DQ'd
              digitalWrite(XtreeSeq2a,LOW);         //Turn on Lane2 Amber Sequence Indicator #1
              digitalWrite(XtreeSeq2b,LOW);         //Turn on Lane2 Amber Sequence Indicator #2
              digitalWrite(XtreeSeq2c,LOW);         //Turn on Lane2 Amber Sequence Indicator #3
            }
            Seq_Time2 = Seq_Time2 + 400;            //Add another 400 ms delay for next sequence
            Ln2_Seq_State = 2;                      //Set Sequence State count to 2 (Amber2 on)
          }
          break;
        case 2:                                     //Turn off amber lights & turn on green light
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Green light
            digitalWrite(XtreeSeq2a,HIGH);          //Turn off Lane2 Amber Sequence Indicator #1
            digitalWrite(XtreeSeq2b,HIGH);          //Turn off Lane2 Amber Sequence Indicator #2
            digitalWrite(XtreeSeq2c,HIGH);          //Turn off Lane2 Amber Sequence Indicator #3
            if (Foul_Flag[1]!=1) digitalWrite(XtreeGrn2,LOW); //Turn on Lane2 Green Go Indicator if not DQ'd
            Ln2_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln2_State = Racing;                     //Set Lane2 race state to Racing
            Start_time[1] = Clk_Time;               //Set Lane2 race start time
            Exp_Time[1] = Clk_Time + max_duration;  //Set Lane2 expiration time till timeout
          }
          break;
        default:
          //Do nothing
          break;
      }
    }
    if (XTmode == 2) {                              //Do Outlaw Xmas Tree lamp sequence
      switch (Ln2_Seq_State) {
        case 0:
          Seq_Time2 = Clk_Time + Dialin_Dly2;       //Set dial in delay for next sequence
          Ln2_Seq_State = 1;                        //Set Sequence State to next case
          break;
        case 1:
          if (Clk_Time > Seq_Time2) {               //Check if time to turn on Green light
            if (Foul_Flag[1]!=1) digitalWrite(XtreeGrn2,LOW); //Turn on Lane2 Green Go Indicator if not DQ'd
            Ln2_Seq_State = 5;                      //Set Sequence State count to 5 (do nothing)
            Ln2_State = Racing;                     //Set Lane2 race state to Racing
            Start_time[1] = Clk_Time;               //Set Lane2 race start time
            Exp_Time[1] = Clk_Time + max_duration;  //Set Lane2 expiration time till timeout
          }
        default:
          //Do nothing
          break;
      }
    }
    if (Foul_Flag[1]==0 && (!Stg2_Flag || Grd2_Flag)) { //Did Lane2 vehicle leave early during countdown?
      React_time[1] = Clk_Time;                     //Get Lane2 reaction time
      Foul_Flag[1] = Foul_Flag[0] + 1;              //Set Lane2 foul flag as 1'st or 2'nd to foul
      if (Foul_Flag[1]==1) digitalWrite(XtreeDsq2,LOW); //Only red-light Lane2 if fouled first (DQ'd)
    }
  }

  //LANE 1 RACING STATE
  if (Ln1_State == Racing) {                        //Lane1 race underway. Check/Wait for Lane1 to finish
    if (L1stage_Flag && Foul_Flag[0]==0) {          //Check & skip if Lane1 already left
      if (!Stg1_Flag || Grd1_Flag) {                //Did Lane1 vehicle leave start line?
        React_time[0] = Clk_Time;                   //Yes...get time
        L1stage_Flag = false;                       //Set Lane1 as not staged
      }
    }
    if (!T06L1_Flag) {                              //Check & skip if Lane1 already tripped 6 Ft sensor
      if (digitalRead(T06_Sense1)==1) {             //Is Lane1 6 Ft sensor tripped?
        T06_time[0] = Clk_Time;                     //Yes...get time
        T06L1_Flag = true;                          //Set Lane1 6 Ft sensed flag
      }
    }
    if (!T66L1_Flag) {                              //Check & skip if Lane1 already tripped 66 Ft sensor
      if (digitalRead(T66_Sense1)==1) {             //Is Lane1 66 Ft sensor tripped?
        T66_time[0] = Clk_Time;                     //Yes...get time
        T66L1_Flag = true;                          //Set Lane1 66 Ft sensed flag
      }
    }
    if (!Trap1_Flag) {                              //Check & skip if Lane1 already tripped Trap sensor
      if (digitalRead(Trap_Sense1)==1) {            //Is Lane1 trap sensor tripped?
        Trap_time[0] = Clk_Time;                    //Yes...get time
        Trap1_Flag = true;                          //Set Lane1 trap sensed flag
      }
    }
    if (!Fin1_Flag) {                               //Check & skip if Lane1 already crossed finish line
      if (digitalRead(Fin_Sense1)==1) {             //Is Car over Lane1 Finish Line?
        Fin_time[0] = Clk_Time;                     //Yes...get time
        Ln1_State = Finished;                       //Set Lane1 state to finished
        Fin1_Flag = true;                           //Set Lane1 finished sensed flag
        digitalWrite(XtreeGrn1,HIGH);               //Turn off Lane1 Green Go Indicator
      }
    }
    if (Clk_Time>Exp_Time[0]) {                     //Has Lane1 race time expired
      Ln1_State = Finished;                         //Set Lane1 state to finished
      digitalWrite(XtreeGrn1,HIGH);                 //Turn off Lane1 Green Go Indicator
    }
  }
  //LANE 2 RACING STATE
  if (Ln2_State == Racing) {                        //Lane2 race underway. Check/Wait for Lane2 to finish
    if (L2stage_Flag && Foul_Flag[1]==0) {          //Check & skip if Lane2 car already left
      if (!Stg2_Flag || Grd2_Flag) {                //Did Lane2 vehicle leave start line?
        React_time[1] = Clk_Time;                   //Yes...get time
        L2stage_Flag = false;                       //Set Lane2 as not staged
      }
    }
    if (!T06L2_Flag) {                              //Check & skip if Lane2 already tripped 6 Ft sensor
      if (digitalRead(T06_Sense2)==1) {             //Is Lane2 6 Ft sensor tripped?
        T06_time[1] = Clk_Time;                     //Yes...get time
        T06L2_Flag = true;                          //Set Lane2 6 Ft sensed flag
      }
    }
    if (!T66L2_Flag) {                              //Check & skip if Lane2 already tripped 66 Ft sensor
      if (digitalRead(T66_Sense2)==1) {             //Is Lane2 66 Ft sensor tripped?
        T66_time[1] = Clk_Time;                     //Yes...get time
        T66L2_Flag = true;                          //Set Lane2 66 Ft sensed flag
      }
    }
    if (!Trap2_Flag) {                              //Check & skip if Lane2 already tripped Trap sensor
      if (digitalRead(Trap_Sense2)==1) {            //Is Lane2 trap sensor tripped?
        Trap_time[1] = Clk_Time;                    //Yes...get time
        Trap2_Flag = true;                          //Set Lane2 trap sensed flag
      }
    }
    if (!Fin2_Flag) {                               //Check & skip if Lane2 already crossed finish line
      if (digitalRead(Fin_Sense2)==1) {             //Is Car over Lane1 Finish Line?
        Fin_time[1] = Clk_Time;                     //Yes...get time
        Ln2_State = Finished;                       //Set Lane2 state to finished
        Fin2_Flag = true;                           //Set Lane2 finished sensed flag
        digitalWrite(XtreeGrn2,HIGH);               //Turn off Lane2 Green Go Indicator
      }
    }
    if (Clk_Time>Exp_Time[1]) {                     //Has Lane2 race time expired
      Ln2_State = Finished;                         //Set Lane2 state to finished
      digitalWrite(XtreeGrn2,HIGH);                 //Turn off Lane2 Green Go Indicator
    }
  }

  //CHECK FOR RACE IN PROGRESS
  if (trackstate == Countdown) {                    //Do only if in coundown state
    if (OneRacer_Flag) {                            //If one racer
      if (Ln1_State == Racing) {                    //Are both cars in racing state?
        trackstate = Racing;                        //Set track state to Racing
        Serial.println(SMsg4);                      //Send Racing state message to PC
      }
    }
    else {                                          //If two racers
      if (Ln1_State == Racing && Ln2_State == Racing) { //Are both cars in racing state?
        trackstate = Racing;                        //Set track state to Racing
        Serial.println(SMsg4);                      //Send Racing state message to PC
      }
    }
  }
  
  //CHECK FOR RACE COMPLETE
  if (trackstate == Racing) {                       //Do only if in Racing state
    if (OneRacer_Flag) {                            //If one racer
      if (Ln1_State==Finished) {
        trackstate = Finished;                      //Set racestate to Finished if Lane1 finished
      }
    }
    else {                                          //If two racers
      if (Ln1_State==Finished && Ln2_State==Finished) {
        trackstate = Finished;                      //set racestate to Finished if both finished
      }
    }
  }

  //RACE IS OVER OR TIME HAS EXPIRED
  if (trackstate==Finished) {                       //Is race complete?
    //Calculate times & speed
    for (i=0; i<Num_Vehicles; i++) {                //Loop through all vehicles
      if (React_time[i]>0) {                        //Check to see if Lane reaction time is valid
        if (Foul_Flag[i]>0) {                       //Did car leave early? (negative reaction time)
          Rtime[i] = float(Start_time[i]-React_time[i])/1000.0; //Calc reaction time in seconds
        }
        else {                                       //Car left after green light (positive reaction time)
          Rtime[i] = float(React_time[i]-Start_time[i])/1000.0; //Calc reaction time in seconds
        }
      }
      if (React_time[i]>0 && Fin_time[i]>React_time[i]) { //Are reaction time & finish time valid?
        Etime[i] = float(Fin_time[i]-React_time[i])/1000.0; //Calc lane elapsed time and convert to seconds
        if (Foul_Flag[i]>0) {                       //Is reaction time negative (Car left before green light)?
          Otime[i] = Etime[i];                      //Yes - overall time is same as elapsed time
        }
        else {                                      //Reaction time is positive (Car left on or after green light)
          Otime[i] = float(Fin_time[i]-Start_time[i])/1000.0; //Calc lane overall time and convert to seconds
        }
      }
      if (Trap_time[i]>0 && Fin_time[i]>Trap_time[i]) { //Are Trap and finish times valid?
        Xtime = Fin_time[i]-Trap_time[i];           //Calc time between trap & finish (milliseconds)
        Speed[i] = float(3600.0/Xtime);             //Calculate lane speed in MPH
      }
    }
    //Determine winner for non-handicap race
    if (!Hdcap_Flag) {                              //Do if not a Handicap race
      if (!OneRacer_Flag) {                         //Do if 2 vehicles racing
        if (Fin_time[0]>0 && Fin_time[1]>0) {       //If both lanes finished
          if (Foul_Flag[0]==0 && Foul_Flag[1]==0) {  //If both lanes didn't foul
            if (Otime[0]<Otime[1]) Win_Flag[0]=true; //Lane1 is winner if Lane1 finished first
            if (Otime[0]>Otime[1]) Win_Flag[1]=true; //Lane2 is winner if Lane2 finished first
            if (Otime[0]==Otime[1]) {               //If both lanes tied
              //DRAW: Use speed to determine winner
              if (Speed[0]>Speed[1]) Win_Flag[0]=true; //Lane1 is winner
              if (Speed[1]>Speed[0]) Win_Flag[1]=true; //Lane2 is winner
            }
          }
          if (Foul_Flag[0]==1 && Foul_Flag[1]==0) { //If only Lane1 fouled
            Win_Flag[1]=true;                       //Lane2 is winner
          }
          if (Foul_Flag[0]==0 && Foul_Flag[1]==1) { //If only Lane2 fouled
            Win_Flag[0]=true;                       //Lane1 is winner
          }
          if (Foul_Flag[0]>0 && Foul_Flag[1]>0) {   //If both lanes fouled
            //The one who fouled first is the loser. (i.e. One with shortest negative React time is winner?
            //Note: If both cars foul only the first DQ light will illuminate.
            if (Foul_Flag[0]==1) Win_Flag[1]=true;  //Lane2 is winner if Lane1 fouled first (DQ'd)
            if (Foul_Flag[1]==1) Win_Flag[0]=true;  //Lane1 is winner if Lane2 fouled first (DQ'd)
          }
        }
        if (Fin_time[0]>0 && Fin_time[1]==0) {      //If only Lane1 finished
          if (Foul_Flag[0]!=1) Win_Flag[0]=true;    //Lane1 is winner if Lane1 not the first to foul (DQ)
        }
        if (Fin_time[0]==0 && Fin_time[1]>0) {      //If only Lane2 finished
          if (Foul_Flag[1]!=1) Win_Flag[1]=true;    //Lane2 is winner if Lane2 not the first to foul (DQ)
        }
      }
      if (OneRacer_Flag) {                          //Do if only one racer (Lane1)
        if (Fin_time[0]>0 && Foul_Flag[0]==0) {     //If Lane1 finished and didn't foul
          Win_Flag[0]=true;                         //Lane1 is winner by default
        }
      }
    }
    //Determine winner for handicap race
    if (Hdcap_Flag) {                               //Do if a Handicap race
      if (Fin_time[0]>0 && Fin_time[1]>0) {         //If both lanes finished
        if (Etime[0]<float(Dialin1/1000.0)) Breakout_Flag[0] = true; //Set Lane1 Breakout flag
        if (Etime[1]<float(Dialin2/1000.0)) Breakout_Flag[1] = true; //Set Lane1 Breakout flag
        if (Foul_Flag[0]==0 && Foul_Flag[1]==0) {   //If both lanes didn't foul
          if (!Breakout_Flag[0] && !Breakout_Flag[1]) { //If both lanes did not breakout
            if (Fin_time[0]<Fin_time[1]) {          //Did Lane1 cross finish line first?
              Win_Flag[0]=true;                     //Lane1 is winner
            }
            if (Fin_time[1]<Fin_time[0]) {          //Did Lane2 cross finish line first?
              Win_Flag[1]=true;                     //Lane2 is winner
            }
            if (Fin_time[0]==Fin_time[1]) {         //Did both lanes cross at same time?
              //Draw: Determine winner by who went fastest
              if (Speed[0]>Speed[1]) Win_Flag[0]=true;  //Lane1 is winner
              if (Speed[1]>Speed[0]) Win_Flag[1]=true;  //Lane2 is winner
            }
          }
          if (Breakout_Flag[0] && !Breakout_Flag[1]) { //Did only Lane1 breakout?
            Win_Flag[1]=true;                       //Lane2 is winner
          }
          if (Breakout_Flag[1] && !Breakout_Flag[0]) { //Did only Lane2 breakout?
            Win_Flag[0]=true;                       //Lane1 is winner
          }
          if (Breakout_Flag[0] && Breakout_Flag[1]) { //If both lanes breakout (Double Breakout)
            Margin[0] = float(Dialin1/1000.0)-Etime[0]; //Determine Lane1 breakout margin
            Margin[1] = float(Dialin2/1000.0)-Etime[1]; //Determine Lane2 breakout margin
            if (Margin[0] < Margin[1]) {            //Is Lane1 margin < Lane2 margin?
              Win_Flag[0]=true;                     //Lane1 is winner
            }
            if (Margin[1] < Margin[0]) {            //Is Lane2 margin < Lane1 margin?
              Win_Flag[1]=true;                     //Lane2 is winner
            }
            if (Margin[0] == Margin[1]) {           //Is Lane1 margin = Lane2 margin?
              if (Fin_time[0]<Fin_time[1]) {        //Did Lane1 cross finish line first?
                Win_Flag[0]=true;                   //Lane1 is winner
              }
              if (Fin_time[1]<Fin_time[0]) {        //Did Lane2 cross finish line first?
                Win_Flag[1]=true;                   //Lane2 is winner
              }
              if (Fin_time[0]==Fin_time[1]) {       //Did both lanes cross at same time?
                //DRAW - Determine winner by who went fastest
                if (Speed[0]>Speed[1]) Win_Flag[0]=true;  //Lane1 is winner
                if (Speed[1]>Speed[0]) Win_Flag[1]=true;  //Lane2 is winner
              }
            }
          } 
        }
        if (Foul_Flag[0]==1 && Foul_Flag[1]==0) {   //If Lane1 only fouled (DQ'd - trumps a Breakout)
          Win_Flag[1]=true;                         //Lane2 is winner
          Win_Flag[0]=false;                        //Lane1 is loser
        }
        if (Foul_Flag[0]==0 && Foul_Flag[1]==1) {   //If Lane2 only fouled (DQ'd - trumps a Breakout)
          Win_Flag[0]=true;                         //Lane1 is winner
          Win_Flag[1]=false;                        //Lane2 is loser
        }
        if (Foul_Flag[0]>0 && Foul_Flag[1]>0) {     //If both lanes fouled
          //The one who fouled first (DQ'd) is the loser
          if (Foul_Flag[0]==1) {                    //If Lane1 fouled first
            Win_Flag[1]=true;                       //Lane2 is winner (Lane1 DQ'd)
            Win_Flag[0]=false;                      //Lane1 is loser
          }
          if (Foul_Flag[1]==1) {                    //If Lane2 fouled first
            Win_Flag[0]=true;                       //Lane1 is winner (Lane2 DQ'd)
            Win_Flag[1]=false;                      //Lane2 is loser
          }
        }
      }
      if (Fin_time[0]>0 && Fin_time[1]==0) {        //If only Lane1 finished
        if (Foul_Flag[0]!=1) {                      //If Lane1 not the first to foul
          Win_Flag[0]=true;                         //Lane1 is winner
        }
      }
      if (Fin_time[0]==0 && Fin_time[1]>0) {        //If only Lane2 finished
          if (Foul_Flag[1]!=1) {                    //If Lane2 not the first to foul
            Win_Flag[1]=true;                       //Lane2 is winner
          }
      }
      if (Fin_time[0]==0 && Fin_time[1]==0) {       //If neither lane finished
        //No winner
      }
    }
    //Set blinking light for winning lane
    if (Win_Flag[0]) {
      L1blink_Flag = true;                          //Set Lane1 blink
      blinkTime = Clk_Time + 15000;                 //Set 15 second blink time
    }
    if (Win_Flag[1]) {
      L2blink_Flag = true;                          //Set Lane2 blink
      blinkTime = Clk_Time + 15000;                 //Set 15 second blink time
    }
    
    //Send Race Results To PC as one long string for each lane
    //Format: Lanex: y z aaaa.aaa bbbb.bbb cccc.ccc dddd.ddd eeee.eee ffff.fff w
    //Where: x=lane#, y=DQ flag, z=Breakout flag, aaaa.aaa=Reaction time, bbbb.bbb=6FT time,
    //       cccc.ccc=66Ft time, dddd.ddd=Trap time, eeee.eee=Finish time, ffff.fff=Speed, w=Win flag
    for (i=0; i<2; i++) {                           //Loop through all lanes
      Serial.print("Lane");                         //Precede string with "Lane" as an identifier
      Serial.print(i+1);                            //Print lane number
      Serial.print(": ");                           //Add a space to separate numbers
      if (Foul_Flag[i]==0) Serial.print("0");       //Print lane foul status (0, 1 or 2)
      if (Foul_Flag[i]==1) Serial.print("1");       //Print lane foul status (0, 1 or 2)
      if (Foul_Flag[i]==2) Serial.print("2");       //Print lane foul status (0, 1 or 2)
      Serial.print(" ");                            //Add a space to separate numbers
      if (Breakout_Flag[i]) Serial.print("1");      //Print lane Breakout status
        else Serial.print("0");
      Serial.print(" ");                            //Add a space to separate numbers
      if (React_time[i]>0) {                        //Check to see if Lane reaction time is valid
        Time = Rtime[i] + 1000.0;                   //Add 1000 to ensure number always has XXXX.XXX format
        Serial.print(Time,3);                       //Print lane reaction time trimmed to 3 places
      }
      else Serial.print("9999.999");                //Print all nines if lane time is invalid
      Serial.print(" ");                            //Add a space to separate numbers
      if (React_time[i]>0 && T06_time[i]>React_time[i]) { //Check to see if lane 6 Ft time is valid
        Time = float(T06_time[i]-React_time[i])/1000.0;  //Calc lane 6 Ft time and convert to seconds
        Time = Time + 1000.0;                       //Add 1000 to ensure number always has XXXX.XXX format
        Serial.print(Time,3);                       //Print lane 6 Ft time trimmed to 3 places
      }
      else Serial.print("9999.999");                //Print all nines if lane 6 Ft time is invalid
      Serial.print(" ");                            //Add a space to separate numbers
      if (React_time[i]>0 && T66_time[i]>React_time[i]) { //Check to see if lane 66 Ft time is valid
        Time = float(T66_time[i]-React_time[i])/1000.0;  //Calc lane 66 Ft time and convert to seconds
        Time = Time + 1000.0;                       //Add 1000 to ensure number always has XXXX.XXX format
        Serial.print(Time,3);                       //Print lane 66 Ft time trimmed to 3 places
      }
      else Serial.print("9999.999");                //Print all nines if lane 66 Ft time is invalid
      Serial.print(" ");                            //Add a space to separate numbers
      if (React_time[i]>0 && Trap_time[i]>React_time[i]) { //Check to see if lane Trap time is valid
        Time = float(Trap_time[i]-React_time[i])/1000.0;  //Calc lane Trap time and convert to seconds
        Time = Time + 1000.0;                       //Add 1000 to ensure number always has XXXX.XXX format
        Serial.print(Time,3);                       //Print lane Trap time trimmed to 3 places
      }
      else Serial.print("9999.999");                //Print all nines if lane Trap time is invalid
      Serial.print(" ");                            //Add a space to separate numbers
      if (React_time[i]>0 && Fin_time[i]>React_time[i]) {  //Check to see if finish time is valid
        Time = float(Fin_time[i]-React_time[i])/1000.0; //Calc lane elapsed time and convert to seconds
        Time = Time + 1000.0;                       //Add 1000 to ensure number always has XXXX.XXX format
        Serial.print(Time,3);                       //Print lane elapsed finish time trimmed to 3 places after decimal
      }
      else Serial.print("9999.999");                //Print all nines if lane finish time is invalid
      Serial.print(" ");
      if (Speed[i]>0.0) {                           //Check to see if speed is valid
        Temp = Speed[i] + 1000.0;                   //Add 1000 to ensure number always has XXXX.XXX format
        Serial.print(Temp,3);                       //Print lane speed trimmed to 3 places
      }
      else Serial.print("9999.999");                //Print all nines if speed is invalid
      Serial.print(" ");
      Serial.println(Win_Flag[i]);                  //Send lane win status & cr/lf to indicate end of the string
      delay(100);                                   //Wait a while to allow display SW to process serial data
    }
    Serial.println(SMsg5);                          //Send Finish state message to PC
    trackstate = NotReady;                          //Set track state to Not Ready
  }
  
  //Blink Green "Go" Indicator to show winner
  if (L1blink_Flag) {                               //Check if Lane1 blink flag is set
    Xtime = Clk_Time & 512;                         //Mask time with binary bit that toggles every 512 ms
    if (Xtime==512) {
      digitalWrite(XtreeGrn1,LOW);                  //Turn on Lane1 Green Go Indicator
    }
    else {
      digitalWrite(XtreeGrn1,HIGH);                 //Turn off Lane1 Green Go Indicator
      if (Clk_Time>blinkTime) L1blink_Flag=false;   //Force end if blink time expired
    }
  }
  if (L2blink_Flag) {                               //Check if Lane2 blink flag is set
    Xtime = Clk_Time & 512;                         //Mask time with binary bit that toggles every 512 ms
    if (Xtime==512) {
      digitalWrite(XtreeGrn2,LOW);                  //Turn on Lane2 Green Go Indicator
    }
    else {
      digitalWrite(XtreeGrn2,HIGH);                 //Turn off Lane2 Green Go Indicator
      if (Clk_Time>blinkTime) L2blink_Flag=false;   //Force end if blink time expired
    }
  }

  //FINISHED STATE
  //Manual pushbutton reset routine. Pushbutton must be held for 0.25 seconds to cause a reset
  if (trackstate==Finished || trackstate==NotReady || trackstate==Ready) { //If not staged or racing
    while(digitalRead(hand_sw)==0) {                //Do while hand switch is pressed
      if (millis() > (Clk_Time + 250)) {            //Has switch been held for 1/4 second?
        Serial.println(SMsg8);                      //Send reset request message to PC
        timer_reset();                              //Call timer reset routine
        break;                                      //exit while loop
      }
      delay(50);                                    //Wait awhile before we loop again
    }
  }
}                                                   //Loop back

//=============================== SUBROUTINES & FUNCTIONS ===============================
//Subroutine: Initialize Racing Variables
void initialize() {  
  for (int n=0; n<2; n++) {                   //Loop through both lanes
    React_time[n] = 0;                        // and reset everything
    T06_time[n] = 0;
    T66_time[n] = 0;
    Trap_time[n] = 0;
    Fin_time[n] = 0;
    Rtime[n] = 0.0;
    Etime[n] = 0.0;
    Otime[n] = 0.0;
    Speed[n] = 0.0;
  }
  Stg1_Flag = false;                          //Reset Lane1 stage sense flag
  Stg2_Flag = false;                          //Reset Lane2 stage sense flag
  Grd1_Flag = false;                          //Reset Lane1 guard sense flag
  Grd2_Flag = false;                          //Reset Lane2 guard sense flag
  T06L1_Flag = false;                         //Reset Lane1 6 Ft sense flag
  T06L2_Flag = false;                         //Reset Lane2 6 Ft sense flag
  T66L1_Flag = false;                         //Reset Lane1 66 Ft sense flag
  T66L2_Flag = false;                         //Reset Lane2 66 Ft sense flag
  Trap1_Flag = false;                         //Reset Lane1 Trap sense flag
  Trap2_Flag = false;                         //Reset Lane2 Trap sense flag
  Fin1_Flag = false;                          //Reset Lane1 Finish sense flag
  Fin2_Flag = false;                          //Reset Lane2 Finish sense flag
  Dialin1 = 0;                                //Reset Lane1 dial-in time to 0
  Dialin2 = 0;                                //Reset Lane2 dial-in time to 0
  Serial.flush();                             //Clear the serial input buffer
  return;
}

//Subroutine: Reset Timer
void timer_reset() {
  Foul_Flag[0] = 0;                           //Reset Lane1 foul flag
  Foul_Flag[1] = 0;                           //Reset Lane2 foul flag
  Win_Flag[0] = false;                        //Set Lane1 win flag to false
  Win_Flag[1] = false;                        //Set Lane2 win flag to false
  Breakout_Flag[0] = false;                   //Set Lane1 breakout flag to false
  Breakout_Flag[1] = false;                   //Set Lane1 breakout flag to false
  L1blink_Flag = false;                       //Set Lane1 blink command to false (off)
  L2blink_Flag = false;                       //Set Lane2 blink command to false (off)
  blinkTime = 0;                              //Reset blink time
  Xtime = 0;                                  //Clear time variable
  Turnoff_All_Lights();                       //Call subroutine to turn off Xmas Tree lights
  Check_Track();                              //Call Check Track subroutine
  if (Track_ready){                           //and only do a reset if Track is ready
    initialize();                             //Call initialize function & reset everything
    trackstate = Ready;                       //Set race state to Ready
    Ln1_State = Ready;                        //Set Lane1 racestate to ready
    Ln2_State = Ready;                        //Set Lane2 racestate to ready
    Serial.println(SMsg1);                    //Send Ready state message to PC
  } 
  else {                                      //Do this if track is not ready
    trackstate = NotReady;                    //Set race state to Not Ready
    if (!Lane1_ready) Ln1_State = NotReady;   //Set Lane1 racestate to not ready
    if (!Lane2_ready) Ln2_State = NotReady;   //Set Lane2 racestate to not ready
    Serial.println(SMsg0);                    //Send Not Ready state message to PC
  }
}

//Subroutine: Check Track - Sets 'Track_ready' to true or false
void Check_Track() {
  Track_ready = true;                         //Default track ready status to true
  Lane1_ready = true;                         //Default Lane1 sensor ready status to true
  Lane2_ready = true;                         //Default Lane2 sensor ready status to true
  error_codes = "";                           //Clear bad sensor string
  //Check Lane1 Sensors
  if (digitalRead(PStg_Sense1)==1) {          //Check if a Lane1 Pre-Stage sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " PStg1";     //Build error string identifying obstructed sensors
  }
  if (digitalRead(Stg_Sense1)==1) {           //Check if a Lane1 Stage sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " Stg1";      //Build error string identifying obstructed sensors
  }
  if (digitalRead(Grd_Sense1)==1) {           //Check if a Lane1 Guard sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " Grd1";      //Build error string identifying obstructed sensors
  }
  if (digitalRead(T06_Sense1)==1) {           //Check if a Lane1 6 Ft track sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " 6ft1";      //Build error string identifying obstructed track sensors
  }
  if (digitalRead(T66_Sense1)==1) {           //Check if a Lane1 66 Ft track sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " 66ft1";     //Build error string identifying obstructed track sensors
  }
  if (digitalRead(Trap_Sense1)==1) {          //Check if a Lane1 trap sensor 1s obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " Trp1";      //Build error string identifying obstructed lane sensors
  }
  if (digitalRead(Fin_Sense1)==1) {           //Check if a Lane1 finish sensor is obstructed
    Lane1_ready = false;                      //If so, set Lane1 sensor ready status to false
    error_codes = error_codes + " Fin1";      //Build error string identifying obstructed lane sensors
  }
  //Check Lane2 Sensors
  if (digitalRead(PStg_Sense2)==1) {          //Check if a Lane2 Pre-Stage sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " PStg2";     //Build error string identifying obstructed sensors
  }
  if (digitalRead(Stg_Sense2)==1) {           //Check if a Lane2 Stage sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " Stg2";      //Build error string identifying obstructed sensors
  }
  if (digitalRead(Grd_Sense2)==1) {           //Check if a Lane2 Guard sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " Grd2";      //Build error string identifying obstructed sensors
  }
  if (digitalRead(T06_Sense2)==1) {           //Check if a Lane2 6 Ft track sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " 6ft2";      //Build error string identifying obstructed track sensors
  }
  if (digitalRead(T66_Sense2)==1) {           //Check if a Lane2 66 Ft track sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " 66ft2";     //Build error string identifying obstructed track sensors
  }
  if (digitalRead(Trap_Sense2)==1) {          //Check if a Lane2 trap sensor 1s obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " Trp2";      //Build error string identifying obstructed lane sensors
  }
  if (digitalRead(Fin_Sense2)==1) {           //Check if a Lane2 finish sensor is obstructed
    Lane2_ready = false;                      //If so, set Lane2 sensor ready status to false
    error_codes = error_codes + " Fin2";      //Build error string identifying obstructed lane sensors
  }
  if (!Lane1_ready || !Lane2_ready) {         //Check if either lane is not ready
    Serial.println(SMsg7 + error_codes);      //If so, send error message to PC
    Track_ready = false;                      //Set track ready status to false
  }
  for (i=0; i<2; i++) {                       //Do lane ready/notready blink routine
    if (!Lane1_ready) digitalWrite(XtreeDsq1,LOW); //Blink Lane1 red light if Lane1 not ready
      else digitalWrite(XtreeGrn1,LOW);            //Blink Lane1 green light if Lane1 ready
    if (!Lane2_ready) digitalWrite(XtreeDsq2,LOW); //Blink Lane2 red light if Lane2 not ready
      else digitalWrite(XtreeGrn2,LOW);            //Blink Lane2 green light if Lane2 ready
    delay(75);
    digitalWrite(XtreeDsq1,HIGH);
    digitalWrite(XtreeDsq2,HIGH);
    digitalWrite(XtreeGrn1,HIGH);
    digitalWrite(XtreeGrn2,HIGH);
    delay(200);
  }
}

//Subroutine - Xmas Tree lamp test
void Lamptest() {
  digitalWrite(XtreePstg1,LOW);               //Turn on Lane1 Pre-Stage Indicator
  digitalWrite(XtreePstg2,LOW);               //Turn on Lane2 Pre-Stage Indicator
  delay(500);
  digitalWrite(XtreePstg1,HIGH);              //Turn off Lane1 Pre-Stage Indicator
  digitalWrite(XtreePstg2,HIGH);              //Turn off Lane2 Pre-Stage Indicator
  digitalWrite(XtreeStg1,LOW);                //Turn on Lane1 Stage Indicator
  digitalWrite(XtreeStg2,LOW);                //Turn on Lane2 Stage Indicator
  delay(500);
  digitalWrite(XtreeStg1,HIGH);               //Turn off Lane1 Stage Indicator
  digitalWrite(XtreeStg2,HIGH);               //Turn off Lane2 Stage Indicator
  digitalWrite(XtreeSeq1a,LOW);               //Turn on Lane1 Amber Sequence Indicator #1
  digitalWrite(XtreeSeq2a,LOW);               //Turn on Lane2 Amber Sequence Indicator #1
  delay(500);
  digitalWrite(XtreeSeq1a,HIGH);              //Turn off Lane1 Amber Sequence Indicator #1
  digitalWrite(XtreeSeq2a,HIGH);              //Turn off Lane2 Amber Sequence Indicator #1
  digitalWrite(XtreeSeq1b,LOW);               //Turn on Lane1 Amber Sequence Indicator #2
  digitalWrite(XtreeSeq2b,LOW);               //Turn on Lane2 Amber Sequence Indicator #2
  delay(500);
  digitalWrite(XtreeSeq1b,HIGH);              //Turn off Lane1 Amber Sequence Indicator #2
  digitalWrite(XtreeSeq2b,HIGH);              //Turn off Lane2 Amber Sequence Indicator #2
  digitalWrite(XtreeSeq1c,LOW);               //Turn on Lane1 Amber Sequence Indicator #3
  digitalWrite(XtreeSeq2c,LOW);               //Turn on Lane2 Amber Sequence Indicator #3
  delay(500);
  digitalWrite(XtreeSeq1c,HIGH);              //Turn off Lane1 Amber Sequence Indicator #3
  digitalWrite(XtreeSeq2c,HIGH);              //Turn off Lane2 Amber Sequence Indicator #3
  digitalWrite(XtreeGrn1,LOW);                //Turn on Lane1 Green Go Indicator
  digitalWrite(XtreeGrn2,LOW);                //Turn on Lane2 Green Go Indicator
  delay(500);
  digitalWrite(XtreeGrn1,HIGH);               //Turn off Lane1 Green Go Indicator
  digitalWrite(XtreeGrn2,HIGH);               //Turn off Lane2 Green Go Indicator
  digitalWrite(XtreeDsq1,LOW);                //Turn on Lane1 Disqualification Indicator
  digitalWrite(XtreeDsq2,LOW);                //Turn on Lane2 Disqualification Indicator
  delay(500);
  digitalWrite(XtreeDsq1,HIGH);               //Turn off Lane1 Disqualification Indicator
  digitalWrite(XtreeDsq2,HIGH);               //Turn off Lane2 Disqualification Indicator
}

//Subroutine - Turn off all Xmas Tree lights
void Turnoff_All_Lights() {
  digitalWrite(XtreePstg1,HIGH);              //Turn off Lane1 white Pre-tage Indicator
  digitalWrite(XtreeStg1,HIGH);               //Turn off Lane1 white Stage Indicator
  digitalWrite(XtreeSeq1a,HIGH);              //Turn off Lane1 Amber Sequence Indicator #1
  digitalWrite(XtreeSeq1b,HIGH);              //Turn off Lane1 Amber Sequence Indicator #2
  digitalWrite(XtreeSeq1c,HIGH);              //Turn off Lane1 Amber Sequence Indicator #3
  digitalWrite(XtreeGrn1,HIGH);               //Turn off Lane1 Green Go Indicator
  digitalWrite(XtreeDsq1,HIGH);               //Turn off Lane1 Red Disq. Indicator
  digitalWrite(XtreePstg2,HIGH);              //Turn off Lane2 white Pre-tage Indicator
  digitalWrite(XtreeStg2,HIGH);               //Turn off Lane2 white Stage Indicator
  digitalWrite(XtreeSeq2a,HIGH);              //Turn off Lane2 Amber Sequence Indicator #1
  digitalWrite(XtreeSeq2b,HIGH);              //Turn off Lane2 Amber Sequence Indicator #2
  digitalWrite(XtreeSeq2c,HIGH);              //Turn off Lane2 Amber Sequence Indicator #3
  digitalWrite(XtreeGrn2,HIGH);               //Turn off Lane2 Green Go Indicator
  digitalWrite(XtreeDsq2,HIGH);               //Turn off Lane2 Red Disq. Indicator
}

//Subroutine - Set Xmas tree dial-In delays
void Set_Dialin_Delays() {
  Hdcap_Flag = false;                         //Preset bracket racing flag to false
  Dialin_Dly1 = 0;                            //Preset Lane1 Xmas tree time delay to zero
  Dialin_Dly2 = 0;                            //Preset Lane2 Xmas tree time delay to zero
  if (Dialin1>0 && Dialin2>0) {               //Do if dial-in times are being used?
    OneRacer_Flag = false;                    //Ensure we're in two-racer mode
    Hdcap_Flag = true;                        //Set handicap flag to true if dial-in times used
    if (Dialin2 < Dialin1) {                  //Do if Lane2 dial-in time < Lane1 dial-in time
      Dialin_Dly2 = Dialin1 - Dialin2;        //Calculate Lane2 Xmas tree sequence time delay
    }
    if (Dialin1 < Dialin2) {                  //Do if Lane1 dial-in time < Lane2 dial-in time
      Dialin_Dly1 = Dialin2 - Dialin1;        //Calculate Lane1 Xmas tree sequence time delay
    }
  }
  Ln1_Seq_State = 0;                          //Initialize Lane1 Sequence State count to 0
  Ln2_Seq_State = 0;                          //Initialize Lane2 Sequence State count to 0
}
